@@ -328,7 +328,9 @@ sub from_clause
     my @from;
 
     my %seen;
-    for my $frag ( $self->_get_from( sort $self->_from_ids() ) )
+    # We lower case  the test for correctly skipping
+    # already-seen tables.
+    for my $frag ( $self->_get_from( sort { lc($a) cmp lc($b) } $self->_from_ids() ) )
     {
         my $join_sql = $frag->sql_with_alias( $dbh, \%seen );
 
